﻿using Connected.Annotations;
using Connected.Services;
using Microsoft.AspNetCore.Components;

namespace Connected.Components;

public partial class BreakpointProvider : UIComponent, IAsyncDisposable
{

	#region Variables
	private Guid _breakPointListenerSubscriptionId;
	
	public Breakpoint Breakpoint { get; private set; } = Breakpoint.Always;

	[Inject] public IBreakpointService Service { get; set; }

	#endregion

	#region Events
	private void SetBreakpointCallback(Breakpoint breakpoint)
	{
		InvokeAsync(() =>
		{
			Breakpoint = breakpoint;
			OnBreakpointChanged.InvokeAsync(breakpoint);
			StateHasChanged();
		}).AndForget();
	}

	[Parameter] public EventCallback<Breakpoint> OnBreakpointChanged { get; set; }
	#endregion

	#region Content
	[Parameter]
	[Category(CategoryTypes.BreakpointProvider.Behavior)]
	public RenderFragment ChildContent { get; set; }
	#endregion

	#region Lifecycle
	protected override async Task OnAfterRenderAsync(bool firstRender)
	{
		await base.OnAfterRenderAsync(firstRender);

		if (firstRender == true)
		{
			var attachResult = await Service.Subscribe(SetBreakpointCallback);
			_breakPointListenerSubscriptionId = attachResult.SubscriptionId;
			Breakpoint = attachResult.Breakpoint;
			await OnBreakpointChanged.InvokeAsync(Breakpoint);
			StateHasChanged();
		}
	}



	public async ValueTask DisposeAsync() => await Service.Unsubscribe(_breakPointListenerSubscriptionId);
	#endregion

}
